@@ -53,37 +53,45 @@ class Cursor(object):
 
         self._result = self.connection.client.sql(sql, parameters,
                                                   bulk_parameters)
+
         if "rows" in self._result:
+            transformed_result = False
             if "col_types" in self._result:
-                rows_to_convert = self._get_rows_to_convert_to_date(self._result["col_types"])
-                self.rows = self._convert_dates_to_datetime(self._result["rows"], rows_to_convert)
-            else:
+                transformed_result = True
+                self.rows = self.result_set_transformed()
+
+            if not transformed_result:
                 self.rows = iter(self._result["rows"])
 
-    @staticmethod
-    def _get_rows_to_convert_to_date(col_types):
+    def result_set_transformed(self):
         """
-        Generates a list of boolean. True if the column is type timestamp (11 - 15)
+        Generator that iterates over each row from the result set
         """
-        return [True if col_type == 11 or col_type == 15 else False for col_type in col_types]
+        rows_to_convert = [True if col_type == 11 or col_type == 15 else False for col_type in
+                           self._result["col_types"]]
+        for row in self._result["rows"]:
+            gen_flags = (flag for flag in rows_to_convert)
+            yield [t_row for t_row in self._transform_date_columns(row, gen_flags)]
 
     @staticmethod
-    def _date_to_datetime(row, rows_to_convert):
+    def _transform_date_columns(row, gen_flags):
         """
-        Converts all values epoch to a datetime object in a given row
+        Generates iterates over each value from a row and converts timestamps to pandas TIMESTAMP
         """
-        return map(lambda value, flag:
-                   datetime.fromtimestamp(float(str(value)[0:10])) if (flag and value is not None) else value,
-                   row,
-                   rows_to_convert)
+        for value in row:
+            try:
+                flag = next(gen_flags)
+            except StopIteration:
+                break
 
-    def _convert_dates_to_datetime(self, rows, rows_to_convert):
-        """
-        Takes a list of rows and map each row to convert date columns to timestamp
-        """
-        return map(lambda x:
-                   self._date_to_datetime(x, (flag for flag in rows_to_convert)),
-                   rows)
+            if not flag or value is None:
+                yield value
+            else:
+                if value < 0:
+                    yield None
+                else:
+                    value = datetime.fromtimestamp(value / 1000)
+                    yield value
 
     def executemany(self, sql, seq_of_parameters):
         """
